/***************************************************************************************
 *
 *  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
 *
 *  By downloading, copying, installing or using the software you agree to this license.
 *  If you do not agree to this license, do not download, install, 
 *  copy or use the software.
 *
 *  Copyright (C) 2010-2014, HappyTime Corporation, all rights reserved.
 *
 *  Any copying, transferring or any other usage is prohibited.
 *
****************************************************************************************/

#ifdef WIN32
#include "stdafx.h"
#endif
#include "ppclient.h"
#include "pp_extern.h"
#if (QT_VERSION >= QT_VERSION_CHECK(5, 1, 0))
#include <QtWidgets/QMessageBox>
#else
#include <QtGui/QMessageBox>
#endif

// event notify callback
void pp_ntf_cb(void * p_userdata, void * sua, PUAEVT evt, void * p_ctx, int len)
{
	ppclient * pDlg = (ppclient *) p_userdata;

	pDlg->onNotify(evt, sua, p_ctx, len);
}

// data callback
void pp_data_cb(void * p_userdata, void * sua, void * p_data, int len)
{
	ppclient * pDlg = (ppclient *) p_userdata;

	pDlg->onData(sua, p_data, len);
}

ppclient::ppclient(QWidget *parent, Qt::WindowFlags flags)
: QDialog(parent, flags)
, m_psua(NULL)
{
	ui.setupUi(this);

	initDialog();
	connSignalSlot();
}

ppclient::~ppclient()
{
	if (m_psua)
	{
		pp_end_call(m_psua);		
		m_psua = NULL;
	}

	pp_stop();
}

void ppclient::initDialog()
{
	pp_start();

	pp_set_ntf_cb(pp_ntf_cb, this);
	pp_set_data_cb(pp_data_cb);

#ifdef ANDROID
    showMaximized();
#endif
}

void ppclient::connSignalSlot()
{
	connect(ui.btnRegister, SIGNAL(clicked()), this, SLOT(slotRegister()));
	connect(ui.btnSend, SIGNAL(clicked()), this, SLOT(slotSend()));
	connect(ui.btnConnect, SIGNAL(clicked()), this, SLOT(slotConnect()));
	connect(ui.btnDisconnect, SIGNAL(clicked()), this, SLOT(slotDisconnect()));
    connect(ui.btnClose, SIGNAL(clicked()), this, SLOT(close()));
	connect(this, SIGNAL(notify(int,void*,void*,int)), this, SLOT(slotNotify(int,void*,void*,int)));
	connect(this, SIGNAL(dataReady()), this, SLOT(slotDataReady()));
}

void ppclient::slotRegister()
{
	QString myid = ui.editMyId->text();
	QString serv = ui.editServer->text();
	unsigned short port = ui.editPort->text().toInt();

	if (myid.isEmpty() || serv.isEmpty())
	{
		return;
	}

	pp_info_set(serv.toStdString().c_str(), port, myid.toStdString().c_str());

	pp_start_reg();
}

void ppclient::slotSend()
{
	QString msg = ui.editSendMsg->toPlainText();
	if (msg.isEmpty())
	{
		return;
	}

	pp_send_data(m_psua, msg.toStdString().c_str(), msg.length());

    QString selfName = pp_get_self_name(m_psua);
    QString remoteName = pp_get_remote_name(m_psua);

    if (selfName.isEmpty())
    {
        selfName = ui.editMyId->text();
    }
    if (remoteName.isEmpty())
    {
        remoteName = ui.editToId->text();
    }
    
    QString info = selfName;
    info += " to ";
    info += remoteName;
    info += " : ";
    info += msg;
    
	ui.editMsg->appendPlainText(info);

	ui.editSendMsg->clear();
}

void ppclient::onNotify(int evt, void * sua, void * p_ctx, int len)
{
	emit notify(evt, sua, p_ctx, len);
}

void ppclient::slotNotify(int evt, void * sua, void * p_ctx, int len)
{
    QString tip;
    
	switch (evt)
	{
	case PUEVT_CALL_IN:
		m_psua = sua;
		tip = QString(tr("Call from ")) + (char *)p_ctx;
		break;

	case PUEVT_CONNECT:
	    tip = QString(tr("Call connect"));
	    break;

	case PUEVT_REG_PASS:
	    tip = QString(tr("Register OK"));
	    break;

	case PUEVT_REG_FAIL:
	    tip = QString(tr("Register Fail"));
	    break;

	case PUEVT_TIMEOUT:
		tip = QString(tr("Call timeout"));
		break;

	case PUEVT_NOUSER:
		tip = QString(tr("User %1 not exist")).arg((char *)p_ctx);
		break;
		
	case PUEVT_DISCONNECT:
		m_psua = NULL;		
	    tip = QString(tr("Call disconnect"));
	    break; 
	}

	ui.editInfo->appendPlainText(tip);
}

void ppclient::slotConnect()
{
	QString toid = ui.editToId->text();
	if (toid.isEmpty())
	{
		return;
	}

	m_psua = pp_make_call(toid.toStdString().c_str());
	if (NULL == m_psua)
	{	
		return;
	}
}

void ppclient::slotDisconnect()
{
	if (m_psua)
	{
		pp_end_call(m_psua);		
		m_psua = NULL;
	}
}

void ppclient::onData(void * sua, void * p_data, int len)
{
    PPDATA data;

    data.pdata = malloc(len+1);
    data.sua = sua;
    data.len = len;
    memcpy(data.pdata, p_data, len);
    ((char *)data.pdata)[len] = '\0';

    QMutexLocker locker(&m_mutex);
    
    m_dataSeq.push_back(data);

    emit dataReady();
}

void ppclient::slotDataReady()
{
    QMutexLocker locker(&m_mutex);

    for (int i = 0; i < m_dataSeq.size(); ++i)
    {
        PPDATA data = m_dataSeq.at(i);

        QString info = pp_get_remote_name(data.sua);
        info += " to ";
        info += pp_get_self_name(data.sua);
        info += " : ";
        info += (char *)data.pdata;
        
    	ui.editMsg->appendPlainText(info);

    	free(data.pdata);
    }

    m_dataSeq.clear();
}





